@@ -15,13 +15,13 @@
 #'   display always normalizes.
 #'
 #' @examples
-#' \dontrun{
-#' img <- read_tif(system.file("img", "Rlogo.tif", package = "ijtiff"))
-#' display(img)
-#' display(img[, , 1, 1]) # first (red) channel, first frame
-#' display(img[, , 2, ]) # second (green) channel, first frame
-#' display(img[, , 3, ]) # third (blue) channel, first frame
-#' display(img, basic = TRUE) # displays first (red) channel, first frame
+#' if (requireNamespace("EBImage")) {
+#'   img <- read_tif(system.file("img", "Rlogo.tif", package = "ijtiff"))
+#'   display(img)
+#'   display(img[, , 1, 1]) # first (red) channel, first frame
+#'   display(img[, , 2, ]) # second (green) channel, first frame
+#'   display(img[, , 3, ]) # third (blue) channel, first frame
+#'   display(img, basic = TRUE) # displays first (red) channel, first frame
 #' }
 #' @export
 display <- function(img, method = NULL, basic = FALSE, normalize = TRUE) {
@@ -50,7 +50,7 @@ display <- function(img, method = NULL, basic = FALSE, normalize = TRUE) {
       if (!methods::is(img, "Image")) img %<>% as_EBImage()
       if (normalize) img %<>% EBImage::normalize()
       if (is.null(method)) {
-        EBImage::display(img)
+        EBImage::display(img, method = "raster")
       } else {
         checkmate::check_string(method)
         method %<>% filesstrings::match_arg(c("browser", "raster"),
