@@ -48,17 +48,9 @@
 #' @seealso [write_tif()]
 #'
 #' @examples
-#' \dontrun{
 #' img <- read_tif(system.file("img", "Rlogo.tif", package = "ijtiff"))
-#' img <- read_tif(system.file("img", "2ch_ij.tif", package = "ijtiff"))
-#' str(img) # we see that `ijtiff` correctly recognises this image's 2 channels
-#' img <- read_tif(system.file("img", "2ch_ij.tif", package = "ijtiff"),
-#'   frames = c(1, 3)
-#' )
-#' }
 #' @export
 read_tif <- function(path, frames = "all", list_safety = "error", msg = TRUE) {
-  err_on_win32bit("read_tif")
   path %<>% prep_path()
   frames %<>% prep_frames()
   withr::local_dir(attr(path, "path_dir"))
@@ -93,7 +85,7 @@ read_tif <- function(path, frames = "all", list_safety = "error", msg = TRUE) {
       stop("`read_tif()` tried to return a list.")
     } else if (list_safety == "warning") {
       warning("`read_tif()` is returning a list.")
-    } else {  # list_safety = "none"
+    } else {
       if (msg) {
         message("Reading a list of images with differing dimensions . . .")
       }
@@ -125,7 +117,6 @@ read_tif <- function(path, frames = "all", list_safety = "error", msg = TRUE) {
 #' @rdname read_tif
 #' @export
 tif_read <- function(path, frames = "all", list_safety = "error", msg = TRUE) {
-  err_on_win32bit("tif_read")
   read_tif(path = path, frames = frames, list_safety = list_safety, msg = msg)
 }
 
@@ -147,16 +138,12 @@ tif_read <- function(path, frames = "all", list_safety = "error", msg = TRUE) {
 #' @seealso [read_tif()]
 #'
 #' @examples
-#' \dontrun{
 #' read_tags(system.file("img", "Rlogo.tif", package = "ijtiff"))
-#' read_tags(system.file("img", "2ch_ij.tif", package = "ijtiff"))
-#' read_tags(system.file("img", "2ch_ij.tif", package = "ijtiff"),
+#' read_tags(system.file("img", "Rlogo-banana.tif", package = "ijtiff"),
 #'   frames = c(2, 4)
 #' )
-#' }
 #' @export
 read_tags <- function(path, frames = 1) {
-  err_on_win32bit("read_tags")
   frames %<>% prep_frames()
   path %<>% prep_path()
   withr::local_dir(attr(path, "path_dir"))
@@ -182,6 +169,5 @@ read_tags <- function(path, frames = 1) {
 #' @rdname read_tags
 #' @export
 tags_read <- function(path, frames = 1) {
-  err_on_win32bit("tags_read")
   read_tags(path = path, frames = frames)
 }
